@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -243,7 +243,7 @@ filter_complete (GMimeFilter *filter, char *in, size_t len, size_t prespace,
 				inleft--;
 			} else if (errno == EINVAL) {
 				/*
-				 * EINVAL  An  incomplete  multibyte sequence has been encoun
+				 * EINVAL  An  incomplete  multibyte sequence has been encoun-
 				 *         tered in the input.
 				 *
 				 * We assume that this can only happen if we've run out of
@@ -304,17 +304,17 @@ filter_reset (GMimeFilter *filter)
 GMimeFilter *
 g_mime_filter_charset_new (const char *from_charset, const char *to_charset)
 {
-	GMimeFilterCharset *new;
+	GMimeFilterCharset *charset;
 	iconv_t cd;
 	
 	cd = g_mime_iconv_open (to_charset, from_charset);
 	if (cd == (iconv_t) -1)
 		return NULL;
 	
-	new = g_object_newv (GMIME_TYPE_FILTER_CHARSET, 0, NULL);
-	new->from_charset = g_strdup (from_charset);
-	new->to_charset = g_strdup (to_charset);
-	new->cd = cd;
+	charset = g_object_new (GMIME_TYPE_FILTER_CHARSET, NULL);
+	charset->from_charset = g_strdup (from_charset);
+	charset->to_charset = g_strdup (to_charset);
+	charset->cd = cd;
 	
-	return (GMimeFilter *) new;
+	return (GMimeFilter *) charset;
 }
