@@ -284,7 +284,7 @@ message_update_addresses (GMimeMessage *message, GMimeParserOptions *options, GM
 			continue;
 		
 		if ((value = g_mime_header_get_raw_value (header))) {
-			if ((list = internet_address_list_parse (options, value))) {
+			if ((list = _internet_address_list_parse (options, value, header->offset))) {
 				internet_address_list_append (addrlist, list);
 				g_object_unref (list);
 			}
@@ -480,7 +480,7 @@ write_headers_to_stream (GMimeObject *object, GMimeFormatOptions *options, GMime
 			header = g_mime_header_list_get_header_at (object->headers, index);
 			offset = g_mime_header_get_offset (header);
 			
-			if (offset >= 0 && offset < body_offset) {
+			if (offset < body_offset) {
 				if (!g_mime_format_options_is_hidden_header (options, header->name)) {
 					if ((nwritten = g_mime_header_write_to_stream (header, options, filtered)) == -1) {
 						g_object_unref (filtered);
@@ -587,7 +587,6 @@ message_write_to_stream (GMimeObject *object, GMimeFormatOptions *options, gbool
 	if (mime_part) {
 		GMimeObjectClass *klass = GMIME_OBJECT_GET_CLASS (mime_part);
 		
-		options = _g_mime_format_options_clone (options, FALSE);
 		mime_part->ensure_newline = ((GMimeObject *) message)->ensure_newline;
 		nwritten = klass->write_to_stream (mime_part, options, TRUE, stream);
 		mime_part->ensure_newline = FALSE;
