@@ -26,7 +26,6 @@
 #include <ctype.h>
 #include <string.h>
 
-#include "gmime-table-private.h"
 #include "gmime-common.h"
 #include "gmime-object.h"
 #include "gmime-stream-mem.h"
@@ -196,12 +195,17 @@ static void
 object_header_changed (GMimeObject *object, GMimeHeader *header)
 {
 	GMimeParserOptions *options = _g_mime_header_list_get_options (object->headers);
+	gboolean can_warn = g_mime_parser_options_get_warning_callback (options) != NULL;
 	GMimeContentDisposition *disposition;
 	GMimeContentType *content_type;
 	const char *name, *value;
 	guint i;
 	
 	name = g_mime_header_get_name (header);
+
+	/* validate header if requested, caches the decoded value */
+	if (G_UNLIKELY (can_warn))
+		g_mime_header_get_value (header);
 	
 	if (g_ascii_strncasecmp (name, "Content-", 8) != 0)
 		return;
@@ -214,13 +218,13 @@ object_header_changed (GMimeObject *object, GMimeHeader *header)
 	switch (i) {
 	case HEADER_CONTENT_DISPOSITION:
 		value = g_mime_header_get_value (header);
-		disposition = g_mime_content_disposition_parse (options, value);
+		disposition = _g_mime_content_disposition_parse (options, value, header->offset);
 		_g_mime_object_set_content_disposition (object, disposition);
 		g_object_unref (disposition);
 		break;
 	case HEADER_CONTENT_TYPE:
 		value = g_mime_header_get_value (header);
-		content_type = g_mime_content_type_parse (options, value);
+		content_type = _g_mime_content_type_parse (options, value, header->offset);
 		_g_mime_object_set_content_type (object, content_type);
 		g_object_unref (content_type);
 		break;
@@ -1102,3 +1106,59 @@ g_mime_object_type_registry_init (void)
 	
 	type_hash = g_hash_table_new (g_mime_strcase_hash, g_mime_strcase_equal);
 }
+
+
+
+GMimeAutocryptHeaderList *
+g_mime_object_get_autocrypt_headers (GMimeObject *mime_part, GDateTime *effective_date,
+				     const char *matchheader, InternetAddressList *addresses,
+				     gboolean keep_incomplete)
+{
+	g_return_val_if_fail (GMIME_IS_OBJECT (mime_part), NULL);
+
+	int i;
+
+	GMimeAutocryptHeaderList *ret = g_mime_autocrypt_header_list_new ();
+	guint count = g_mime_autocrypt_header_list_add_missing_addresses (ret, addresses);
+	if (!count)
+		return ret;
+
+	/* scan for Autocrypt headers whose addr= attribute matches
+	 * the From: header. */
+	
+	GMimeHeaderList *headers = g_mime_object_get_header_list(mime_part);
+	for (i = 0; i < g_mime_header_list_get_count (headers); i++) {
+		GMimeHeader *header = g_mime_header_list_get_header_at (headers, i);
+		if (g_ascii_strcasecmp (matchheader, header->name) == 0) {
+			GMimeAutocryptHeader *ah = g_mime_autocrypt_header_new_from_string (g_mime_header_get_value (header));
+			if (!ah || ! g_mime_autocrypt_header_is_complete (ah))
+				goto done;
+			g_mime_autocrypt_header_set_effective_date (ah, effective_date);
+			GMimeAutocryptHeader *prev = g_mime_autocrypt_header_list_get_header_for_address (ret, ah->address);
+			if (!prev) /* not a valid address (was not in From:) */
+				goto done;
+			if (g_mime_autocrypt_header_is_complete (prev)) {
+				/* this is a duplicate (we use effective_date=NULL as an internal marker for this) */
+				g_mime_autocrypt_header_set_effective_date (prev, NULL);
+			} else {
+				g_mime_autocrypt_header_clone (prev, ah);
+			}
+		done:
+			if (ah)
+				g_object_unref (ah);
+		}
+			
+	}
+	for (i = 0; i < g_mime_autocrypt_header_list_get_count (ret); i++) {
+		GMimeAutocryptHeader *ah = g_mime_autocrypt_header_list_get_header_at (ret, i);
+		/* drop keydata from duplicates */
+		if (ah->effective_date == NULL) {
+			g_mime_autocrypt_header_set_keydata (ah, NULL);
+			g_mime_autocrypt_header_set_effective_date (ah, effective_date);
+		}
+	}
+
+	if (!keep_incomplete)
+		g_mime_autocrypt_header_list_remove_incomplete (ret);
+	return ret;
+}
