@@ -794,7 +794,7 @@ g_mime_param_list_encode (GMimeParamList *list, GMimeFormatOptions *options, gbo
 			/* we need to do special rfc2184 parameter wrapping */
 			size_t maxlen = GMIME_FOLD_LEN - (nlen + 6);
 			char *inend;
-			int i = 0;
+			int n = 0;
 			
 			inptr = value;
 			inend = value + vlen;
@@ -816,7 +816,7 @@ g_mime_param_list_encode (GMimeParamList *list, GMimeFormatOptions *options, gbo
 						ptr = q;
 				}
 				
-				if (i != 0) {
+				if (n != 0) {
 					g_string_append_c (str, ';');
 					
 					if (fold) {
@@ -830,7 +830,7 @@ g_mime_param_list_encode (GMimeParamList *list, GMimeFormatOptions *options, gbo
 					used = 1;
 				}
 				
-				g_string_append_printf (str, "%s*%d*=", param->name, i++);
+				g_string_append_printf (str, "%s*%d*=", param->name, n++);
 				g_string_append_len (str, inptr, (size_t) (ptr - inptr));
 				used += (str->len - here);
 				
@@ -1055,7 +1055,7 @@ decode_rfc2184_param (const char **in, char **namep, int *part, gboolean *encode
 
 static gboolean
 decode_param (GMimeParserOptions *options, const char **in, char **namep, char **valuep, int *id,
-	      const char **rfc2047_charset, gboolean *encoded, GMimeParamEncodingMethod *method)
+	      const char **rfc2047_charset, gboolean *encoded, GMimeParamEncodingMethod *method, gint64 offset)
 {
 	GMimeRfcComplianceMode mode = g_mime_parser_options_get_parameter_compliance_mode (options);
 	gboolean is_rfc2184 = FALSE;
@@ -1080,7 +1080,7 @@ decode_param (GMimeParserOptions *options, const char **in, char **namep, char *
 				 * this, we should handle this case.
 				 */
 				
-				if ((val = _g_mime_utils_header_decode_text (options, value, rfc2047_charset))) {
+				if ((val = _g_mime_utils_header_decode_text (options, value, rfc2047_charset, offset))) {
 					*method = GMIME_PARAM_ENCODING_METHOD_RFC2047;
 					g_free (value);
 					value = val;
@@ -1110,6 +1110,7 @@ decode_param (GMimeParserOptions *options, const char **in, char **namep, char *
 		return TRUE;
 	}
 	
+	_g_mime_parser_options_warn (options, offset, GMIME_WARN_INVALID_PARAMETER, name);
 	g_free (value);
 	g_free (name);
 	
@@ -1311,8 +1312,9 @@ rfc2184_param_new (char *name, char *value, int id, gboolean encoded)
 }
 
 static GMimeParamList *
-decode_param_list (GMimeParserOptions *options, const char *in)
+decode_param_list (GMimeParserOptions *options, const char *in, gint64 offset)
 {
+	gboolean can_warn = g_mime_parser_options_get_warning_callback (options) != NULL;
 	struct _rfc2184_param *rfc2184, *list, *t;
 	char *name, *value, *charset, *lang;
 	GMimeParamEncodingMethod method;
@@ -1337,7 +1339,7 @@ decode_param_list (GMimeParserOptions *options, const char *in)
 	
 	do {
 		/* invalid format? */
-		if (!decode_param (options, &inptr, &name, &value, &id, &rfc2047_charset, &encoded, &method)) {
+		if (!decode_param (options, &inptr, &name, &value, &id, &rfc2047_charset, &encoded, &method, offset)) {
 			skip_cfws (&inptr);
 			
 			if (*inptr == ';')
@@ -1411,6 +1413,29 @@ decode_param_list (GMimeParserOptions *options, const char *in)
 		g_free (rfc2184);
 		rfc2184 = t;
 	}
+
+	if (can_warn) {
+		GMimeParam *p;
+		guint j;
+		
+		for (i = 0; i < params->array->len; i++) {
+			param = params->array->pdata[i];
+		
+			for (j = i + 1; j < params->array->len; j++) {
+				p = params->array->pdata[j];
+				
+				if (g_ascii_strcasecmp (param->name, p->name) != 0)
+					continue;
+				
+				if (strcmp (param->value, p->value) != 0)
+					_g_mime_parser_options_warn (options, offset, GMIME_CRIT_CONFLICTING_PARAMETER, param->name);
+				else
+					_g_mime_parser_options_warn (options, offset, GMIME_WARN_DUPLICATED_PARAMETER, param->name);
+				
+				break;
+			}
+		}
+	}
 	
 	return params;
 }
@@ -1427,8 +1452,14 @@ decode_param_list (GMimeParserOptions *options, const char *in)
  **/
 GMimeParamList *
 g_mime_param_list_parse (GMimeParserOptions *options, const char *str)
+{
+	return _g_mime_param_list_parse (options, str, -1);
+}
+
+GMimeParamList *
+_g_mime_param_list_parse (GMimeParserOptions *options, const char *str, gint64 offset)
 {
 	g_return_val_if_fail (str != NULL, NULL);
 	
-	return decode_param_list (options, str);
+	return decode_param_list (options, str, offset);
 }
