@@ -847,19 +847,6 @@ next_alloc_size (size_t n)
 	priv->headerleft -= len;                                          \
 } G_STMT_END
 
-static inline gboolean
-is_7bit_clean (const gchar *str)
-{
-	while (*str != '\0') {
-		if ((*str & 0x80) != 0)
-			return FALSE;
-		
-		str++;
-	}
-	
-	return TRUE;
-}
-
 static void
 header_parse (GMimeParser *parser, GMimeParserOptions *options)
 {
@@ -889,7 +876,7 @@ header_parse (GMimeParser *parser, GMimeParserOptions *options)
 	
 	if (*inptr != ':') {
 		/* ignore invalid headers */
-		_g_mime_parser_options_warn (options, priv->header_offset, GMIME_WARN_INVALID_HEADER, priv->headerbuf);
+		_g_mime_parser_options_warn (options, priv->header_offset, GMIME_CRIT_INVALID_HEADER_NAME, priv->headerbuf);
 		w(g_warning ("Invalid header at %lld: '%s'",
 			     (long long) priv->header_offset,
 			     priv->headerbuf));
@@ -922,7 +909,7 @@ header_parse (GMimeParser *parser, GMimeParserOptions *options)
 		priv->header_cb (parser, header->name, header->raw_value,
 				 header->offset, priv->user_data);
 	
-	if (can_warn && (!is_7bit_clean (header->name) || !g_utf8_validate (header->raw_value, -1, NULL)))
+	if (can_warn && (g_mime_utils_text_is_8bit ((const unsigned char *) header->name, strlen (header->name)) || !g_utf8_validate (header->raw_value, -1, NULL)))
 		_g_mime_parser_options_warn (options, header->offset, GMIME_WARN_UNENCODED_8BIT_HEADER, header->name);
 }
 
@@ -975,9 +962,25 @@ has_content_headers (GPtrArray *headers)
 	return FALSE;
 }
 
+static void
+warn_invalid_header (GMimeParser *parser, GMimeParserOptions *options, const char *start, const char *inptr, const char *inend)
+{
+	struct _GMimeParserPrivate *priv = parser->priv;
+	const char *eoln = inptr;
+	char *header;
+	
+	while (eoln < inend && *eoln != '\r' && *eoln != '\n')
+		eoln++;
+	
+	header = g_strndup (start, eoln - start);
+	_g_mime_parser_options_warn (options, priv->header_offset, GMIME_CRIT_INVALID_HEADER_NAME, header);
+	g_free (header);
+}
+
 static int
 parser_step_headers (GMimeParser *parser, GMimeParserOptions *options)
 {
+	gboolean can_warn = g_mime_parser_options_get_warning_callback (options) != NULL;
 	struct _GMimeParserPrivate *priv = parser->priv;
 	gboolean eoln, valid = TRUE, fieldname = TRUE;
 	gboolean continuation = FALSE;
@@ -1058,9 +1061,11 @@ parser_step_headers (GMimeParser *parser, GMimeParserOptions *options)
 				}
 				
 				if (!valid) {
-					if (priv->format == GMIME_FORMAT_MBOX &&
-					    is_mbox_marker (start, (size_t) (inptr - start), FALSE))
+					if (priv->format == GMIME_FORMAT_MBOX && is_mbox_marker (start, (size_t) (inptr - start), FALSE)) {
+						if (can_warn)
+							warn_invalid_header (parser, options, start, inptr, inend);
 						goto next_message;
+					}
 					
 					if (priv->headers->len > 0) {
 						if (priv->state == GMIME_PARSER_STATE_MESSAGE_HEADERS) {
@@ -1068,12 +1073,16 @@ parser_step_headers (GMimeParser *parser, GMimeParserOptions *options)
 								/* probably the start of the content,
 								 * a broken mailer didn't terminate the
 								 * headers with an empty line. *sigh* */
+								if (can_warn)
+									warn_invalid_header (parser, options, start, inptr, inend);
 								goto content_start;
 							}
 						} else if (has_content_headers (priv->headers)) {
 							/* probably the start of the content,
 							 * a broken mailer didn't terminate the
 							 * headers with an empty line. *sigh* */
+							if (can_warn)
+								warn_invalid_header (parser, options, start, inptr, inend);
 							goto content_start;
 						}
 					} else if (priv->state == GMIME_PARSER_STATE_MESSAGE_HEADERS) {
@@ -1081,6 +1090,8 @@ parser_step_headers (GMimeParser *parser, GMimeParserOptions *options)
 						 * headers, but remain lenient with lines starting with
 						 * "From " or ">From ". */
 						if (!is_mbox_marker (start, (size_t) (inptr - start), TRUE)) {
+							if (can_warn)
+								warn_invalid_header (parser, options, start, inptr, inend);
 							priv->state = GMIME_PARSER_STATE_ERROR;
 							return -1;
 						}
@@ -1627,6 +1638,37 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, Bounda
 	}
 }
 
+static void
+check_header_conflict (GMimeParserOptions *options, GMimeObject *object, const Header *header)
+{
+	const GMimeHeader *existing;
+
+	if ((existing = g_mime_header_list_get_header (object->headers, header->name)) != NULL) {
+		if (strcmp (existing->raw_value, header->raw_value) != 0)
+			_g_mime_parser_options_warn (options, header->offset, GMIME_CRIT_CONFLICTING_HEADER, header->name);
+		else
+			_g_mime_parser_options_warn (options, header->offset, GMIME_WARN_DUPLICATED_HEADER, header->name);
+	}
+}
+
+static int
+compare_header (const void *a, const void *b)
+{
+	return g_ascii_strcasecmp ((const gchar *) a, * (const gchar **) b);
+}
+
+/* headers which may exist only once according to RFC 5322, Sect. 3.6 (keep the list sorted) */
+static const char *rfc5322_single_hdr[] = {
+	"bcc", "cc", "date", "from", "in-reply-to", "message-id", "references", "reply-to", "sender", "subject", "to"
+};
+
+static void
+check_repeated_header (GMimeParserOptions *options, GMimeObject *object, const Header *header)
+{
+	if (bsearch (header->name, rfc5322_single_hdr, G_N_ELEMENTS (rfc5322_single_hdr), sizeof (char *), compare_header))
+		check_header_conflict (options, object, header);
+}
+
 static void
 parser_scan_message_part (GMimeParser *parser, GMimeParserOptions *options, GMimeMessagePart *mpart, BoundaryType *found)
 {
@@ -1634,6 +1676,7 @@ parser_scan_message_part (GMimeParser *parser, GMimeParserOptions *options, GMim
 	ContentType *content_type;
 	GMimeMessage *message;
 	GMimeObject *object;
+	gboolean can_warn;
 	Header *header;
 	guint i;
 	
@@ -1689,13 +1732,17 @@ parser_scan_message_part (GMimeParser *parser, GMimeParserOptions *options, GMim
 	
 	message = g_mime_message_new (FALSE);
 	((GMimeObject *) message)->ensure_newline = FALSE;
+	_g_mime_header_list_set_options (((GMimeObject *) message)->headers, options);
 	message->marker = priv->preheader;
 	priv->preheader = NULL;
 	
+	can_warn = g_mime_parser_options_get_warning_callback (options) != NULL;
 	for (i = 0; i < priv->headers->len; i++) {
 		header = priv->headers->pdata[i];
 		
 		if (g_ascii_strncasecmp (header->name, "Content-", 8) != 0) {
+			if (can_warn)
+				check_repeated_header (options, (GMimeObject *) message, header);
 			_g_mime_object_append_header ((GMimeObject *) message, header->name, header->raw_name,
 						      header->raw_value, header->offset);
 		}
@@ -1714,19 +1761,6 @@ parser_scan_message_part (GMimeParser *parser, GMimeParserOptions *options, GMim
 	g_object_unref (message);
 }
 
-static void
-check_content_header_conflict (GMimeParserOptions *options, GMimeObject *object, const Header *header)
-{
-	const GMimeHeader *existing;
-	
-	if ((existing = g_mime_header_list_get_header (object->headers, header->name)) != NULL) {
-		if (strcmp (existing->raw_value, header->raw_value) != 0)
-			_g_mime_parser_options_warn (options, header->offset, GMIME_CRIT_CONFLICTING_CONTENT_HDR, header->name);
-		else
-			_g_mime_parser_options_warn (options, header->offset, GMIME_WARN_DUPLICATED_CONTENT_HDR, header->name);
-	}
-}
-
 static GMimeObject *
 parser_construct_leaf_part (GMimeParser *parser, GMimeParserOptions *options, ContentType *content_type, gboolean toplevel, BoundaryType *found)
 {
@@ -1751,7 +1785,7 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeParserOptions *options, Co
 		header = priv->headers->pdata[i];
 		
 		if (!toplevel || !g_ascii_strncasecmp (header->name, "Content-", 8)) {
-			check_content_header_conflict (options, object, header);
+			check_header_conflict (options, object, header);
 			_g_mime_object_append_header (object, header->name, header->raw_name,
 						      header->raw_value, header->offset);
 		}
@@ -1895,7 +1929,7 @@ parser_construct_multipart (GMimeParser *parser, GMimeParserOptions *options, Co
 		header = priv->headers->pdata[i];
 		
 		if (!toplevel || !g_ascii_strncasecmp (header->name, "Content-", 8)) {
-			check_content_header_conflict (options, object, header);
+			check_header_conflict (options, object, header);
 			
 			if (!g_ascii_strcasecmp (header->name, "Content-Type"))
 				ctype_offset = header->offset;
@@ -2013,6 +2047,7 @@ parser_construct_message (GMimeParser *parser, GMimeParserOptions *options)
 	GMimeObject *object;
 	BoundaryType found;
 	const char *inptr;
+	gboolean can_warn;
 	Header *header;
 	char *endptr;
 	guint i;
@@ -2033,6 +2068,7 @@ parser_construct_message (GMimeParser *parser, GMimeParserOptions *options)
 	((GMimeObject *) message)->ensure_newline = FALSE;
 	_g_mime_header_list_set_options (((GMimeObject *) message)->headers, options);
 	
+	can_warn = g_mime_parser_options_get_warning_callback (options) != NULL;
 	for (i = 0; i < priv->headers->len; i++) {
 		header = priv->headers->pdata[i];
 		
@@ -2047,6 +2083,8 @@ parser_construct_message (GMimeParser *parser, GMimeParserOptions *options)
 		}
 		
 		if (g_ascii_strncasecmp (header->name, "Content-", 8) != 0) {
+			if (can_warn)
+				check_repeated_header (options, (GMimeObject *) message, header);
 			_g_mime_object_append_header ((GMimeObject *) message, header->name, header->raw_name,
 						      header->raw_value, header->offset);
 		}
