@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -123,16 +123,9 @@ g_mime_stream_cat_init (GMimeStreamCat *stream, GMimeStreamCatClass *klass)
 static void
 g_mime_stream_cat_finalize (GObject *object)
 {
-	GMimeStreamCat *cat = (GMimeStreamCat *) object;
-	struct _cat_node *n, *nn;
+	GMimeStream *stream = (GMimeStream *) object;
 	
-	n = cat->sources;
-	while (n != NULL) {
-		nn = n->next;
-		g_object_unref (n->stream);
-		g_free (n);
-		n = nn;
-	}
+	stream_close (stream);
 	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -641,11 +634,11 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 	} while (n != NULL);
 	
 	d(fprintf (stderr, "returning a substream containing multiple source streams\n"));
-	cat = g_object_newv (GMIME_TYPE_STREAM_CAT, 0, NULL);
+	cat = g_object_new (GMIME_TYPE_STREAM_CAT, NULL);
 	/* Note: we could pass -1 as bound_end, it should Just
 	 * Work(tm) but setting absolute bounds is kinda
 	 * nice... */
-	g_mime_stream_construct (GMIME_STREAM (cat), 0, subend);
+	g_mime_stream_construct ((GMimeStream *) cat, 0, subend);
 	
 	while (streams != NULL) {
 		s = streams->next;
@@ -684,7 +677,7 @@ g_mime_stream_cat_new (void)
 {
 	GMimeStream *stream;
 	
-	stream = g_object_newv (GMIME_TYPE_STREAM_CAT, 0, NULL);
+	stream = g_object_new (GMIME_TYPE_STREAM_CAT, NULL);
 	g_mime_stream_construct (stream, 0, -1);
 	
 	return stream;
