@@ -51,7 +51,7 @@
 #include <errno.h>
 
 #ifdef LIBIDN
-#include <idna.h>
+#include <idn2.h>
 #endif
 
 #include "gmime-utils.h"
@@ -857,9 +857,9 @@ g_mime_utils_generate_message_id (const char *fqdn)
 	g_string_append_c (msgid, '@');
 	
 #ifdef LIBIDN
-	if (idna_to_ascii_8z (fqdn, &ascii, 0) == IDNA_SUCCESS) {
+	if (idn2_to_ascii_8z (fqdn, &ascii, 0) == IDN2_OK) {
 		g_string_append (msgid, ascii);
-		free (ascii);
+		idn2_free (ascii);
 	} else {
 		g_string_append (msgid, fqdn);
 	}
@@ -902,7 +902,7 @@ need_quotes (const char *string)
 			inptr++;
 		else if (*inptr == '"')
 			quoted = !quoted;
-		else if (!quoted && (is_tspecial (*inptr) || *inptr == '.'))
+		else if (!quoted && is_special (*inptr))
 			return TRUE;
 		
 		if (*inptr)
@@ -916,12 +916,12 @@ need_quotes (const char *string)
  * g_mime_utils_quote_string:
  * @str: input string
  *
- * Quotes @string as needed according to the rules in rfc2045.
+ * Quotes @string as needed according to the rules in rfc2822.
  * 
  * Returns: an allocated string containing the escaped and quoted (if
  * needed to be) input string. The decision to quote the string is
- * based on whether or not the input string contains any 'tspecials'
- * as defined by rfc2045.
+ * based on whether or not the input string contains any 'specials'
+ * as defined by rfc2822.
  **/
 char *
 g_mime_utils_quote_string (const char *str)
@@ -1473,10 +1473,12 @@ rfc2047_token_new_encoded_word (const char *word, size_t len)
 }
 
 static rfc2047_token *
-tokenize_rfc2047_phrase (GMimeParserOptions *options, const char *in, size_t *len)
+tokenize_rfc2047_phrase (GMimeParserOptions *options, const char *in, size_t *len, gint64 offset)
 {
+	gboolean can_warn = g_mime_parser_options_get_warning_callback (options) != NULL;
 	rfc2047_token list, *lwsp, *token, *tail;
 	register const char *inptr = in;
+	gboolean has_specials = FALSE;
 	GMimeRfcComplianceMode mode;
 	gboolean encoded = FALSE;
 	const char *text, *word;
@@ -1507,12 +1509,17 @@ tokenize_rfc2047_phrase (GMimeParserOptions *options, const char *in, size_t *le
 				 * have been merged with other
 				 * words. */
 				
+				has_specials = FALSE;
+				
 				if (!strncmp (inptr, "=?", 2)) {
 					inptr += 2;
 					
 					/* skip past the charset (if one is even declared, sigh) */
 					while (*inptr && *inptr != '?') {
-						ascii = ascii && is_ascii (*inptr);
+						if (!is_atom (*inptr)) {
+							ascii = ascii && is_ascii (*inptr);
+							has_specials = TRUE;
+						}
 						inptr++;
 					}
 					
@@ -1524,12 +1531,16 @@ tokenize_rfc2047_phrase (GMimeParserOptions *options, const char *in, size_t *le
 					
 					/* find the end of the rfc2047 encoded word token */
 					while (*inptr && strncmp (inptr, "?=", 2) != 0) {
-						ascii = ascii && is_ascii (*inptr);
+						if (!is_atom (*inptr)) {
+							ascii = ascii && is_ascii (*inptr);
+							has_specials = TRUE;
+						}
 						inptr++;
 					}
 					
 					if (*inptr == '\0') {
 						/* didn't find an end marker... */
+						has_specials = FALSE;
 						inptr = word + 2;
 						ascii = TRUE;
 						
@@ -1551,6 +1562,9 @@ tokenize_rfc2047_phrase (GMimeParserOptions *options, const char *in, size_t *le
 			
 			n = (size_t) (inptr - word);
 			if ((token = rfc2047_token_new_encoded_word (word, n))) {
+				if (can_warn && has_specials)
+					_g_mime_parser_options_warn (options, offset, GMIME_WARN_INVALID_RFC2047_HEADER_VALUE, in);
+				
 				/* rfc2047 states that you must ignore all
 				 * whitespace between encoded words */
 				if (!encoded && lwsp != NULL) {
@@ -1608,10 +1622,12 @@ tokenize_rfc2047_phrase (GMimeParserOptions *options, const char *in, size_t *le
 }
 
 static rfc2047_token *
-tokenize_rfc2047_text (GMimeParserOptions *options, const char *in, size_t *len)
+tokenize_rfc2047_text (GMimeParserOptions *options, const char *in, size_t *len, gint64 offset)
 {
+	gboolean can_warn = g_mime_parser_options_get_warning_callback (options) != NULL;
 	rfc2047_token list, *lwsp, *token, *tail;
 	register const char *inptr = in;
+	gboolean has_specials = FALSE;
 	GMimeRfcComplianceMode mode;
 	gboolean encoded = FALSE;
 	const char *text, *word;
@@ -1638,12 +1654,16 @@ tokenize_rfc2047_text (GMimeParserOptions *options, const char *in, size_t *len)
 			ascii = TRUE;
 			
 			if (G_LIKELY (mode == GMIME_RFC_COMPLIANCE_LOOSE)) {
+				has_specials = FALSE;
+				
 				if (!strncmp (inptr, "=?", 2)) {
 					inptr += 2;
 					
 					/* skip past the charset (if one is even declared, sigh) */
 					while (*inptr && *inptr != '?') {
 						ascii = ascii && is_ascii (*inptr);
+						if (is_lwsp (*inptr))
+							has_specials = TRUE;
 						inptr++;
 					}
 					
@@ -1656,11 +1676,14 @@ tokenize_rfc2047_text (GMimeParserOptions *options, const char *in, size_t *len)
 					/* find the end of the rfc2047 encoded word token */
 					while (*inptr && strncmp (inptr, "?=", 2) != 0) {
 						ascii = ascii && is_ascii (*inptr);
+						if (is_lwsp (*inptr))
+							has_specials = TRUE;
 						inptr++;
 					}
 					
 					if (*inptr == '\0') {
 						/* didn't find an end marker... */
+						has_specials = FALSE;
 						inptr = word + 2;
 						ascii = TRUE;
 						
@@ -1672,8 +1695,7 @@ tokenize_rfc2047_text (GMimeParserOptions *options, const char *in, size_t *len)
 				non_rfc2047:
 					/* stop if we encounter a possible rfc2047 encoded
 					 * token even if it's inside another word, sigh. */
-					while (*inptr && !is_lwsp (*inptr) &&
-					       strncmp (inptr, "=?", 2) != 0) {
+					while (*inptr && !is_lwsp (*inptr) && strncmp (inptr, "=?", 2) != 0) {
 						ascii = ascii && is_ascii (*inptr);
 						inptr++;
 					}
@@ -1687,6 +1709,9 @@ tokenize_rfc2047_text (GMimeParserOptions *options, const char *in, size_t *len)
 			
 			n = (size_t) (inptr - word);
 			if ((token = rfc2047_token_new_encoded_word (word, n))) {
+				if (can_warn && has_specials)
+					_g_mime_parser_options_warn (options, offset, GMIME_WARN_INVALID_RFC2047_HEADER_VALUE, in);
+				
 				/* rfc2047 states that you must ignore all
 				 * whitespace between encoded words */
 				if (!encoded && lwsp != NULL) {
@@ -1868,6 +1893,7 @@ rfc2047_decode_tokens (GMimeParserOptions *options, rfc2047_token *tokens, size_
  * @text: header text to decode
  * @options: (nullable): a #GMimeParserOptions or %NULL
  * @charset: (optional): if non-%NULL, this will be set to the charset used in the rfc2047 encoded-word tokens
+ * @offset: header offset, only used for reporting a #GMimeParserWarning
  *
  * Decodes an rfc2047 encoded 'text' header.
  *
@@ -1875,7 +1901,7 @@ rfc2047_decode_tokens (GMimeParserOptions *options, rfc2047_token *tokens, size_
  * header.
  **/
 char *
-_g_mime_utils_header_decode_text (GMimeParserOptions *options, const char *text, const char **charset)
+_g_mime_utils_header_decode_text (GMimeParserOptions *options, const char *text, const char **charset, gint64 offset)
 {
 	rfc2047_token *tokens;
 	char *decoded;
@@ -1888,7 +1914,7 @@ _g_mime_utils_header_decode_text (GMimeParserOptions *options, const char *text,
 		return g_strdup ("");
 	}
 	
-	tokens = tokenize_rfc2047_text (options, text, &len);
+	tokens = tokenize_rfc2047_text (options, text, &len, offset);
 	decoded = rfc2047_decode_tokens (options, tokens, len, charset);
 	rfc2047_token_list_free (tokens);
 	
@@ -1909,7 +1935,7 @@ _g_mime_utils_header_decode_text (GMimeParserOptions *options, const char *text,
 char *
 g_mime_utils_header_decode_text (GMimeParserOptions *options, const char *text)
 {
-	return _g_mime_utils_header_decode_text (options, text, NULL);
+	return _g_mime_utils_header_decode_text (options, text, NULL, -1);
 }
 
 
@@ -1918,6 +1944,7 @@ g_mime_utils_header_decode_text (GMimeParserOptions *options, const char *text)
  * @phrase: header to decode
  * @options: (nullable): a #GMimeParserOptions or %NULL
  * @charset: (optional): if non-%NULL, this will be set to the charset used in the rfc2047 encoded-word tokens
+ * @offset: header offset, only used for reporting a #GMimeParserWarning
  *
  * Decodes an rfc2047 encoded 'phrase' header.
  *
@@ -1925,7 +1952,7 @@ g_mime_utils_header_decode_text (GMimeParserOptions *options, const char *text)
  * header.
  **/
 char *
-_g_mime_utils_header_decode_phrase (GMimeParserOptions *options, const char *phrase, const char **charset)
+_g_mime_utils_header_decode_phrase (GMimeParserOptions *options, const char *phrase, const char **charset, gint64 offset)
 {
 	rfc2047_token *tokens;
 	char *decoded;
@@ -1938,7 +1965,7 @@ _g_mime_utils_header_decode_phrase (GMimeParserOptions *options, const char *phr
 		return g_strdup ("");
 	}
 	
-	tokens = tokenize_rfc2047_phrase (options, phrase, &len);
+	tokens = tokenize_rfc2047_phrase (options, phrase, &len, offset);
 	decoded = rfc2047_decode_tokens (options, tokens, len, charset);
 	rfc2047_token_list_free (tokens);
 	
@@ -1959,7 +1986,7 @@ _g_mime_utils_header_decode_phrase (GMimeParserOptions *options, const char *phr
 char *
 g_mime_utils_header_decode_phrase (GMimeParserOptions *options, const char *phrase)
 {
-	return _g_mime_utils_header_decode_phrase (options, phrase, NULL);
+	return _g_mime_utils_header_decode_phrase (options, phrase, NULL, -1);
 }
 
 
@@ -2616,7 +2643,7 @@ g_mime_utils_structured_header_fold (GMimeParserOptions *options, GMimeFormatOpt
 	while (*value && is_lwsp (*value))
 		value++;
 	
-	tokens = tokenize_rfc2047_phrase (options, value, &len);
+	tokens = tokenize_rfc2047_phrase (options, value, &len, -1);
 	folded = header_fold_tokens (format, field, value, len, tokens, TRUE, TRUE);
 	g_free (field);
 	
@@ -2648,7 +2675,7 @@ _g_mime_utils_structured_header_fold (GMimeParserOptions *options, GMimeFormatOp
 	if (value == NULL)
 		return g_strdup ("\n");
 	
-	tokens = tokenize_rfc2047_phrase (options, value, &len);
+	tokens = tokenize_rfc2047_phrase (options, value, &len, -1);
 	
 	return header_fold_tokens (format, field, value, len, tokens, TRUE, FALSE);
 }
@@ -2689,7 +2716,7 @@ g_mime_utils_unstructured_header_fold (GMimeParserOptions *options, GMimeFormatO
 	while (*value && is_lwsp (*value))
 		value++;
 	
-	tokens = tokenize_rfc2047_text (options, value, &len);
+	tokens = tokenize_rfc2047_text (options, value, &len, -1);
 	folded = header_fold_tokens (format, field, value, len, tokens, FALSE, TRUE);
 	g_free (field);
 	
@@ -2720,7 +2747,7 @@ _g_mime_utils_unstructured_header_fold (GMimeParserOptions *options, GMimeFormat
 	if (value == NULL)
 		return g_strdup ("\n");
 	
-	tokens = tokenize_rfc2047_text (options, value, &len);
+	tokens = tokenize_rfc2047_text (options, value, &len, -1);
 	
 	return header_fold_tokens (format, field, value, len, tokens, FALSE, FALSE);
 }
